﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.ProgramSynthesis {
  [StorableClass]
  [Item("AfterCrossoverOperator", "Performs an action after the crossover operator is applied.")]
  public class AfterManipulatorOperator<T> : EvolutionTrackingOperator<T>, IManipulatorOperator<T> where T : class,IItem {
    private const string ChildParameterName = "Child";

    public ILookupParameter<T> ChildParameter {
      get { return (ILookupParameter<T>)Parameters[ChildParameterName]; }
    }

    protected AfterManipulatorOperator(AfterManipulatorOperator<T> original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new AfterManipulatorOperator<T>(this, cloner);
    }
    [StorableConstructor]
    protected AfterManipulatorOperator(bool deserializing) : base(deserializing) { }

    public AfterManipulatorOperator() {
      Parameters.Add(new LookupParameter<T>(ChildParameterName));
    }

    public override IOperation Apply() {
      return base.Apply();
    }
  }
}
